/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence;

import com.topcoder.db.connectionfactory.DBConnectionFactory;
import com.topcoder.db.connectionfactory.DBConnectionFactoryImpl;

import com.topcoder.document.index.wordsource.WordSource;
import com.topcoder.document.index.wordsource.textfile.TextFileWordSource;

import com.topcoder.util.config.ConfigManager;

import java.io.File;
import java.io.FileWriter;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;


/**
 * <p>
 * Accuracy test helper.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TestHelper {
    /**
     * <p>
     * Represents the namespace for DB Connection Factory component.
     * </p>
     */
    public static final String DB_FACTORY_NAMESPACE = "com.topcoder.db.connectionfactory.DBConnectionFactoryImpl";

    /**
     * <p>
     * Represents the path of config file for DB Connection Factory component.
     * </p>
     */
    public static final String DB_FACTORY_CONFIGFILE = "test_files/dbfactory_test.xml";

    /**
     * <p>
     * Represents the namespace for Simple Access DB Index solution.
     * </p>
     */
    public static final String SIMPLE_ACCESS_NAMESPACE =
        "com.topcoder.document.index.persistence.impl.db.SimpleAccessDBIndexPersistence";

    /**
     * <p>
     * Represents the path of config file for Simple Access DB Index solution.
     * </p>
     */
    public static final String SIMPLE_ACCESS_CONFIGFILE = "test_files/SimpleAccessDBIndexPersistence.xml";

    /**
     * <p>
     * Represents the namespace for Fast Access DB Index solution.
     * </p>
     */
    public static final String FAST_ACCESS_NAMESPACE =
        "com.topcoder.document.index.persistence.impl.db.FastAccessDBIndexPersistence";

    /**
     * <p>
     * Represents the path of config file for Fast Access DB Index solution.
     * </p>
     */
    public static final String FAST_ACCESS_CONFIGFILE = "test_files/FastAccessDBIndexPersistence.xml";

    /**
     * <p>
     * Represents the namespace for XML Index solution.
     * </p>
     */
    public static final String XML_INDEX_NAMESPACE =
        "com.topcoder.document.index.persistence.impl.xml.XmlIndexPersistence";

    /**
     * <p>
     * Represents the path of config file for XML Index solution.
     * </p>
     */
    public static final String XML_INDEX_CONFIGFILE = "test_files/XmlIndexPersistence.xml";

    /**
     * <p>
     * Represents the database connection name for Simple Access DB Index solution..
     * </p>
     */
    public static final String SIMLE_ACCESS_CONNECTION_NAME = "SimpleSolution";

    /**
     * <p>
     * Represents the database connection name for Fast Access DB Index solution..
     * </p>
     */
    public static final String FAST_ACCESS_CONNECTION_NAME = "FastSolution";

    /**
     * <p>
     * Represents the delimiters used to separate words.
     * </p>
     */
    public static final String[] DELIMITERS = {" ", "\n", "\t", "\r", ",", "\"", ".", "!", "-", "?", "(", ")"};

    /**
     * <P>
     * Represents the simple text file path.
     * </P>
     */
    public static final String SIMPLE_TEXT = "test_files/simple.txt";

    /**
     * <P>
     * Represents the huge text file path.
     * </P>
     */
    public static final String HUGE_TEXT = "test_files/huge.txt";

    /**
     * <P>
     * Represents the medium text file path.
     * </P>
     */
    public static final String MEDIUM_TEXT = "test_files/medium.txt";

    /**
     * <P>
     * Represents the xml persistence root path.
     * </P>
     */
    public static final String XML_PERSISTENCE_ROOT = "test_files/resource";

    /**
     * <p>
     * The DBConnectionFactory instance for testing.
     * </p>
     */
    private static DBConnectionFactory dbFactory = null;

    /**
     * <p>
     * This private constructor prevents to create a new instance.
     * </p>
     */
    private TestHelper() {
    }

    /**
     * <p>
     * Use the given file to config the given namespace the format of the config file is
     * ConfigManager.CONFIG_XML_FORMAT.
     * </p>
     *
     * @param namespace use the namespace to load config information to ConfigManager
     * @param fileName config file to set up environment
     *
     * @throws Exception when any exception occurs
     */
    public static void loadSingleXMLConfig(String namespace, String fileName)
        throws Exception {
        //set up environment
        ConfigManager config = ConfigManager.getInstance();
        File file = new File(fileName);

        //config namespace
        if (config.existsNamespace(namespace)) {
            config.removeNamespace(namespace);
        }

        config.add(namespace, file.getCanonicalPath(),
            ConfigManager.CONFIG_XML_FORMAT);
    }

    /**
     * <p>
     * Use the given file to config the given namespace the format of the config file is
     * ConfigManager.CONFIG_MULTIPLE_XML_FORMAT.
     * </p>
     *
     * @param namespace use the namespace to load config information to ConfigManager
     * @param fileName config file to set up environment
     *
     * @throws Exception when any exception occurs
     */
    public static void loadMultipleXMLConfig(String namespace, String fileName)
        throws Exception {
        //set up environment
        ConfigManager config = ConfigManager.getInstance();
        File file = new File(fileName);

        //config namespace
        if (config.existsNamespace(namespace)) {
            config.removeNamespace(namespace);
        }

        config.add(namespace, file.getCanonicalPath(),
            ConfigManager.CONFIG_MULTIPLE_XML_FORMAT);
    }

    /**
     * <p>
     * Remove the given namespace in the ConfigManager.
     * </p>
     *
     * @param namespace namespace use to remove the config information in ConfigManager
     *
     * @throws Exception when any exception occurs
     */
    public static void clearConfigFile(String namespace)
        throws Exception {
        ConfigManager config = ConfigManager.getInstance();

        //clear the environment
        if (config.existsNamespace(namespace)) {
            config.removeNamespace(namespace);
        }
    }

    /**
     * <p>
     * Clears all the data in tables for simple access solution.
     * </p>
     *
     * @param conn the connection instance for database operation.
     */
    public static void clearSimpleSolutionTables(Connection conn) {
        String[] tables = new String[] {"DOCUMENT_COLLECTION_DOCUMENT_XREF", "WORD_POSITION", "WORD",
            "DELIMITER", "DOCUMENT", "DOCUMENT_COLLECTION"};

        clearTables(conn, tables);
    }

    /**
     * <p>
     * Clears all the data in tables for fast access solution.
     * </p>
     *
     * @param conn the connection instance for database operation.
     */
    public static void clearFastSolutionTables(Connection conn) {
        String[] tables = new String[] {"DOCUMENT_COLLECTION_DOCUMENT_XREF", "WORD_DATA", "DELIMITER",
            "DOCUMENT", "DOCUMENT_COLLECTION"};

        clearTables(conn, tables);
    }

    /**
     * <p>
     * Clears all the xml files in xml persistence root path and recreate the empty document-usage.xml file.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public static void clearXmlSolutionFiles() throws Exception {
        File dir = new File(XML_PERSISTENCE_ROOT);
        File[] files = dir.listFiles();

        for (int i = 0; i < files.length; i++) {
            if (files[i].getName().endsWith(".xml")) {
                files[i].delete();
            }
        }

        StringBuffer buffer = new StringBuffer();
        buffer.append("<?xml version=\"1.0\"?>\r\n");
        buffer.append("<documents>\r\n");
        buffer.append("</documents>\r\n");

        FileWriter writer = null;

        try {
            writer = new FileWriter(new File(dir, "document_usage.xml"));
            writer.write(buffer.toString());
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }

    /**
     * <p>
     * Clears all the data in tables specified by the given tables array.
     * </p>
     *
     * @param conn the connection instance for database operation.
     * @param tables contains table names
     */
    private static void clearTables(Connection conn, String[] tables) {
        Statement stmt = null;

        try {
            stmt = conn.createStatement();

            for (int i = 0; i < tables.length; i++) {
                stmt.executeUpdate("DELETE FROM " + tables[i]);
            }
        } catch (SQLException se) {
            se.printStackTrace();

            // ignore
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    //ignore
                }
            }

            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException e) {
                    //ignore
                }
            }
        }
    }

    /**
     * <p>
     * Returns the DBConnectionFactory instance for testing.
     * </p>
     *
     * @return the DBConnectionFactory instance for testing.
     *
     * @throws Exception to JUnit.
     */
    public static DBConnectionFactory getDBFactory() throws Exception {
        if (dbFactory == null) {
            dbFactory = new DBConnectionFactoryImpl(DB_FACTORY_NAMESPACE);
        }

        return dbFactory;
    }

    /**
     * <p>
     * Returns a WordSource instance for testing.
     * The underlying text file is a simple text file.
     * </p>
     *
     * @return a WordSource instance for testing.
     * @throws Exception to JUnit
     */
    public static WordSource getSimpleWordSource() throws Exception {
        return new TextFileWordSource(TestHelper.SIMPLE_TEXT,
            TestHelper.DELIMITERS);
    }

    /**
     * <p>
     * Returns a WordSource instance for testing.
     * The underlying text file is a medium text file.
     * </p>
     *
     * @return a WordSource instance for testing.
     * @throws Exception to JUnit
     */
    public static WordSource getMediumWordSource() throws Exception {
        return new TextFileWordSource(TestHelper.MEDIUM_TEXT,
            TestHelper.DELIMITERS);
    }

    /**
     * <p>
     * Returns a WordSource instance for testing.
     * The underlying text file is a huge text file.
     * </p>
     *
     * @return a WordSource instance for testing.
     * @throws Exception to JUnit
     */
    public static WordSource getHugeWordSource() throws Exception {
        return new TextFileWordSource(TestHelper.HUGE_TEXT,
            TestHelper.DELIMITERS);
    }

    /**
     * <p>
     * Returns a SAXParset instance for parsing xml file.
     * </p>
     *
     * @return a SAXParset instance for parsing xml file.
     * @throws Exception to JUnit.
     */
    public static SAXParser getSAXParser() throws Exception {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setNamespaceAware(false);
        factory.setValidating(false);

        return factory.newSAXParser();
    }
}
