/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl;

import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.TestHelper;
import com.topcoder.document.index.wordsource.WordSource;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * <p>
 * Unit test cases for DocumentIndexPersistenceUtil.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DocumentIndexPersistenceUtilTests extends TestCase {
    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(DocumentIndexPersistenceUtilTests.class);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkNull(Object,String) method.
     * It test the case when a not null object is passed in and expects success.
     * </p>
     */
    public void testCheckNull_NotNullObject() {
        DocumentIndexPersistenceUtil.checkNull("", "test");
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkNull(Object,String) method.
     * It test the case when a null object is passed in and expects IllegalArgumentException.
     * </p>
     */
    public void testCheckNull_NullObject() {
        try {
            DocumentIndexPersistenceUtil.checkNull(null, "test");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkString(String,String) method.
     * It test the case when a not null and not empty string is passed in and expects success.
     * </p>
     */
    public void testCheckString_NotNullNotEmptyString() {
        DocumentIndexPersistenceUtil.checkString("test", "test");
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkString(String,String) method.
     * It test the case when a null string is passed in and expects IllegalArgumentException.
     * </p>
     */
    public void testCheckString_NullString() {
        try {
            DocumentIndexPersistenceUtil.checkString(null, "test");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkString(String,String) method.
     * It test the case when an empty string is passed in and expects IllegalArgumentException.
     * </p>
     */
    public void testCheckString_EmptyString() {
        try {
            DocumentIndexPersistenceUtil.checkString("", "test");
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(int,String,String,int) for accuracy.
     * </p>
     */
    public void testCheckInt1_GE() {
        DocumentIndexPersistenceUtil.checkInt(3, "data", "GE", 3);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(int,String,String,int) for accuracy.
     * </p>
     */
    public void testCheckInt1_GT() {
        DocumentIndexPersistenceUtil.checkInt(5, "data", "GT", 3);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(int,String,String,int) for failure.
     * It test the case when the two int values don't satisfy the operator and expects
     * IllegalArgumentException.
     * </p>
     */
    public void testCheckInt1_GTFail() {
        try {
            DocumentIndexPersistenceUtil.checkInt(3, "data", "GT", 3);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(int,String,String,int) for failure.
     * It test the case when the two int values don't satisfy the operator and expects
     * IllegalArgumentException.
     * </p>
     */
    public void testCheckInt1_GEFail() {
        try {
            DocumentIndexPersistenceUtil.checkInt(2, "data", "GE", 3);
            fail("IllegalArgumentException expected.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(int,String,String,int) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCheckInt2_GE() throws Exception {
        DocumentIndexPersistenceUtil.checkInt("3", "data", "GE", 3);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(String,String,String,int) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCheckInt2_GT() throws Exception {
        DocumentIndexPersistenceUtil.checkInt("5", "data", "GT", 3);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(String,String,String,int) for failure.
     * It test the case when the two int values don't satisfy the operator and expects
     * PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCheckInt2_GTFail() throws Exception {
        try {
            DocumentIndexPersistenceUtil.checkInt("3", "data", "GT", 3);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(String,String,String,int) for failure.
     * It test the case when the two int values don't satisfy the operator and expects
     * PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCheckInt2_GEFail() throws Exception {
        try {
            DocumentIndexPersistenceUtil.checkInt("2", "data", "GE", 3);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#checkInt(String,String,String,int) for failure.
     * It test the case when the string is not a valid integer and expects PersistenceConfigurationException.
     * </p>
     */
    public void testCheckInt_PersistenceConfigurationException() {
        try {
            DocumentIndexPersistenceUtil.checkInt("a32", "fileSizeLimit", "GE",
                12);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#getString(String,String,boolean) method.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetString() throws Exception {
        TestHelper.loadSingleXMLConfig(TestHelper.XML_INDEX_NAMESPACE,
            TestHelper.XML_INDEX_CONFIGFILE);

        String value = DocumentIndexPersistenceUtil.getString(TestHelper.XML_INDEX_NAMESPACE,
                "FileSizeLimit", true);
        assertEquals("Failed to get correct property value from configuration manager.",
            "1", value);

        TestHelper.clearConfigFile(TestHelper.XML_INDEX_NAMESPACE);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#getString(String,String,boolean) for failure.
     * It tests the case that property is missing and it's optional.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testGetString_OptionalProperty() throws Exception {
        TestHelper.loadSingleXMLConfig(TestHelper.XML_INDEX_NAMESPACE,
            TestHelper.XML_INDEX_CONFIGFILE);

        String value = DocumentIndexPersistenceUtil.getString(TestHelper.XML_INDEX_NAMESPACE,
                "NotExistProperty", false);

        assertNull("Expects null property value.", value);

        TestHelper.clearConfigFile(TestHelper.XML_INDEX_NAMESPACE);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#getString(String,String,boolean) for failure.
     * It tests the case that property is missing but it's required, expects PersistenceConfigurationException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testGetString_RequiredProperty() throws Exception {
        TestHelper.loadSingleXMLConfig(TestHelper.XML_INDEX_NAMESPACE,
            TestHelper.XML_INDEX_CONFIGFILE);

        try {
            DocumentIndexPersistenceUtil.getString(TestHelper.XML_INDEX_NAMESPACE,
                "NotExistProperty", true);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException e) {
            //good
        }

        TestHelper.clearConfigFile(TestHelper.XML_INDEX_NAMESPACE);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#createObject(Class,String,Class[],Object[]) method.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCreateObject_NoArguments() throws Exception {
        DocumentIndexPersistenceUtil.createObject(String.class,
            "java.lang.String", null, null);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#createObject(Class,String,Class[],Object[]) method.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testCreateObject_HasArguments() throws Exception {
        DocumentIndexPersistenceUtil.createObject(StringBuffer.class,
            "java.lang.StringBuffer", new Class[] {String.class},
            new Object[] {"test"});
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#createObject(Class,String,Class[],Object[]) method.
     * It tests the case when the class name can't be found in class path and expects PersistenceConfigurationException.
     * </p>
     *
     */
    public void testCreateObject_NotClassFound() {
        try {
            DocumentIndexPersistenceUtil.createObject(StringBuffer.class,
                "com.tc.StringBuffer", null, null);
            fail("PersistenceConfigurationException expected.");
        } catch (PersistenceConfigurationException pce) {
            //good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#serializeObject(Object) and
     * DocumentIndexPersistenceUtil#deSerializeObject(byte[]) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testSerialize_DeSerialize() throws Exception {
        String value = "value";
        byte[] data = DocumentIndexPersistenceUtil.serializeObject(value);
        Object obj = DocumentIndexPersistenceUtil.deSerializeObject(data);

        assertEquals("Failed to serialize/deserialize an object.", obj, value);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#serializeObject(Object) for failure.
     * It tests the case when the given objec is not serializable and expects IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testSerializeObject_IndexPersistenceException()
        throws Exception {
        Thread thread = Thread.currentThread();

        try {
            DocumentIndexPersistenceUtil.serializeObject(thread);
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#deSerializeObject(byte) for failure.
     * It tests the case when an arbitrary byte array is used for deserializing and expects IndexPersistenceException.
     * </p>
     *
     * @throws Exception to JUit.
     */
    public void testDeSerializeObject_IndexPersistenceException()
        throws Exception {
        byte[] data = "value".getBytes();

        try {
            DocumentIndexPersistenceUtil.deSerializeObject(data);
            fail("IndexPersistenceException expected.");
        } catch (IndexPersistenceException e) {
            //good
        }
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#getDigestBasedId(WordSourceId) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetDigestBasedId() throws Exception {
        WordSource source1 = TestHelper.getSimpleWordSource();
        WordSource source2 = TestHelper.getMediumWordSource();

        String id1 = DocumentIndexPersistenceUtil.getDigestBasedId(source1.getId());
        String id2 = DocumentIndexPersistenceUtil.getDigestBasedId(source2.getId());

        assertFalse("Failed to generate unique id for WordSourceId instance.",
            id1.equals(id2));
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#base64Encode(byte[]) and
     * DocumentIndexPersistenceUtil#base64Decode(String) for accuracy.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testBase64Encode_Decode() throws Exception {
        byte[] rawData = "testBase64Encode_Decode".getBytes();
        // encode the data
        String encodedData = DocumentIndexPersistenceUtil.base64Encode(rawData);

        // and then decode it
        byte[] data = DocumentIndexPersistenceUtil.base64Decode(encodedData);

        // expects the encode/decode works well.
        assertEquals("Expects same length.", data.length, rawData.length);

        boolean isEquals = true;

        for (int i = 0; i < rawData.length; i++) {
            if (rawData[i] != data[i]) {
                isEquals = false;

                break;
            }
        }

        assertTrue("Failed to encode/decode a byte array using BASE64 algorithm correctly.",
            isEquals);
    }

    /**
     * <p>
     * Tests DocumentIndexPersistenceUtil#hexCode(String) and DocumentIndexPersistenceUtil#hexDecode(String)
     * for accuracy.
     * </p>
     */
    public void testHexCode_HexDecode() {
        StringBuffer value = new StringBuffer();
        value.append("\n");
        value.append("\r");
        value.append("\t");
        value.append("\"");

        // encode the data
        String encodedData = DocumentIndexPersistenceUtil.hexCode(value.toString());
        // decode it
        String decodedData = DocumentIndexPersistenceUtil.hexDecode(encodedData);

        // expects the encode/decode works well.
        assertEquals("Failed to encode/decode a String using hex code correctly.",
            decodedData, value.toString());
    }
}
