/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.persistence.TestHelper;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.SAXParser;


/**
 * <p>
 * Unit test cases for CollectionIndexHandler.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class CollectionIndexHandlerTests extends TestCase {
    /**
     * <p>
     * Represents the path of the xml file for testing.
     * </p>
     */
    private static final String XML_TEST_FILE = "test_files/handler_xml/document_collection_sample.xml";

    /**
     * <p>
     * CollectionIndexHandler instance for testing.
     * </p>
     */
    private CollectionIndexHandler handler;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void setUp() throws Exception {
        SAXParser parser = TestHelper.getSAXParser();
        handler = new CollectionIndexHandler();
        parser.parse(XML_TEST_FILE, handler);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        handler = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(CollectionIndexHandlerTests.class);
    }

    /**
     * <p>
     * Tests CollectionIndexHandler#getCollectionId() for accuracy.
     * </p>
     */
    public void testGetCollectionId() {
        assertEquals("Failed to get correct collection id.", "37684",
            handler.getCollectionId());
    }

    /**
     * <p>
     * Tests CollectionIndexHandler#getDocumentIdList() for accuracy.
     * </p>
     */
    public void testGetDocumentIdList() {
        List documentIds = new ArrayList();
        documentIds.add("16572");
        documentIds.add("46537");
        assertEquals("Failed to get correct document index id owned by this collection index.",
            documentIds, handler.getDocumentIdList());
    }
}
