/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.persistence.TestHelper;
import com.topcoder.document.index.wordsource.WordSourceId;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.text.CollationKey;
import java.text.Collator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.xml.parsers.SAXParser;


/**
 * <p>
 * Unit test cases for DocumentIndexHandler.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DocumentIndexHandlerTests extends TestCase {
    /**
     * <p>
     * Represents the path of the xml file for testing.
     * </p>
     */
    private static final String XML_TEST_FILE = "test_files/handler_xml/document_index_sample.xml";

    /**
     * <p>
     * DocumentIndexHandler instance for testing.
     * </p>
     */
    private DocumentIndexHandler handler;

    /**
     * <p>
     * Setup test environment.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    protected void setUp() throws Exception {
        SAXParser parser = TestHelper.getSAXParser();
        handler = new DocumentIndexHandler(null);
        parser.parse(XML_TEST_FILE, handler);
    }

    /**
     * <p>
     * Tears down test environment.
     * </p>
     *
     */
    protected void tearDown() {
        handler = null;
    }

    /**
     * <p>
     * Return all tests.
     * </p>
     *
     * @return all tests
     */
    public static Test suite() {
        return new TestSuite(DocumentIndexHandlerTests.class);
    }

    /**
     * <p>
     * Tests DocumentIndexHandler#getWordSourceId() for accuracy.
     * Expects the word-source-data in document_index_sample.xml file in test_files/resource directory
     * be parsed correctly.
     * </p>
     *
     * @throws Exception to JUnit.
     */
    public void testGetWordSourceId() throws Exception {
        WordSourceId wordSourceId = handler.getWordSourceId();


        // class name
        assertEquals("Failed to get correct class name.", "class name",
            wordSourceId.getSourceClassName());

        // Locale information
        Locale locale = new Locale("en", "US", "POSIX");
        assertEquals("Failed to get correct locale information.", locale,
            wordSourceId.getSourceLocale());

        // delimiters
        List delimiters = new ArrayList();
        delimiters.add(" ");
        delimiters.add("\n");
        delimiters.add("\t");
        delimiters.add("\r");
        assertEquals("Failed to get correct delimiter list.", delimiters,
            Arrays.asList(wordSourceId.getDelimiters()));
    }

    /**
     * <p>
     * Tests DocumentIndexHandler#getWords() for accuracy.
     * Expects the word-index in document_index_sample.xml file in test_files/resource directory
     * be parsed correctly.
     * </p>
     */
    public void testGetWords() {
        Map words = handler.getWords();

        assertEquals("Expects the length of words is one.", 1, words.size());

        Locale locale = new Locale("en", "US", "POSIX");
        Collator collator = Collator.getInstance(locale);


        CollationKey key = collator.getCollationKey("Hello");
        assertTrue("Expects Hello is contained as a word.", words.containsKey(key));

        List positions = new ArrayList();
        positions.add(new Integer(24));
        positions.add(new Integer(56));
        assertEquals("Failed to get correct position list.", positions,
            words.get(key));
    }

    /**
     * <p>
     * Tests DocumentIndexHandler#getDocumentIndexId() for accuracy.
     * </p>
     */
    public void testGetDocumentIndexId() {
        assertEquals("Failed to get correct document index id.", "16527",
            handler.getDocumentIndexId());
    }
}
